#ifndef _INCLUDE_BLUETOOTH_H
#define _INCLUDE_BLUETOOTH_H

#include <devctl.h>
#include <sys/socket.h>

#define AF_BLUETOOTH    31

#define BTPROTO_L2CAP   1

typedef struct _bdaddr {
    unsigned char   b[6];
} __attribute__ ((packed)) bdaddr_t;

#define BDADDR_ANY (&(bdaddr_t) {{0, 0, 0, 0, 0, 0}})

typedef struct _inq_res {
    bdaddr_t                bdaddr;
    unsigned char           page_scan_rep_mode;
    unsigned char           page_scan_period_mode;
    unsigned char           page_scan_mode;
    unsigned int            class_of_device;
    unsigned short          clock_offset;
} inq_res_t;

typedef struct _dcmd_blue_inquiry {
    int         num_responses;
    inq_res_t   result[0];
} dcmd_blue_inquiry_t;
typedef struct _dcmd_blue_remote_name {
    bdaddr_t        bdaddr;
    unsigned char   remote_name[248];
} dcmd_blue_remote_name_t;

#define _DCMD_BLUE_BASE     48
#define DCMD_BLUE_INQUIRY       __DIOTF(_DCMD_NET, _DCMD_BLUE_BASE + 0, dcmd_blue_inquiry_t)
#define DCMD_BLUE_REMOTE_NAME   __DIOTF(_DCMD_NET, _DCMD_BLUE_BASE + 1, dcmd_blue_remote_name_t)
#define DCMD_BLUE_LOCAL_NAME    __DIOF( _DCMD_NET, _DCMD_BLUE_BASE + 2, unsigned char)
#define DCMD_BLUE_CHANGE_NAME   __DIOT( _DCMD_NET, _DCMD_BLUE_BASE + 3, unsigned char)
#define DCMD_BLUE_GET_BDADDR    __DIOF( _DCMD_NET, _DCMD_BLUE_BASE + 4, bdaddr_t)

/* Byte order conversions */
#if __BYTE_ORDER == __LITTLE_ENDIAN
#define htobs(d)  (d)
#define htobl(d)  (d)
#define btohs(d)  (d)
#define btohl(d)  (d)
#elif __BYTE_ORDER == __BIG_ENDIAN
#define htobs(d)  bswap_16(d)
#define htobl(d)  bswap_32(d)
#define btohs(d)  bswap_16(d)
#define btohl(d)  bswap_32(d)
#else
#error "Unknown byte order"
#endif

/* Copy, swap, convert BD Address */
static inline int bacmp(bdaddr_t *ba1, bdaddr_t *ba2)
{
	return memcmp(ba1, ba2, sizeof(bdaddr_t));
}
static inline void bacpy(bdaddr_t *dst, bdaddr_t *src)
{
	memcpy(dst, src, sizeof(bdaddr_t));
}

void baswap(bdaddr_t *dst, bdaddr_t *src);
bdaddr_t *strtoba(char *str);
char     *batostr(bdaddr_t *ba);
int  ba2str(bdaddr_t *ba, char *str);
int  str2ba(char *str, bdaddr_t *ba);

int  bt_error(uint16_t code);
char *bt_compidtostr(int id);

#endif
