#ifndef _INCLUDE_HCI_H
#define _INCLUDE_HCI_H

#include <bluetooth/bluetooth.h>

typedef struct _hci_event {
    unsigned char   event;
    unsigned char   len;
    unsigned char   param[0];
} __attribute__ ((packed)) hci_event_t;

typedef struct _hci_acldata {
    unsigned short  handle:12;
    unsigned char   boundary:2;
    unsigned char   broadcast:2;
    unsigned short  len;
    unsigned char   data[0];
} __attribute__ ((packed)) hci_acldata_t;

#define L2CAP_MTU           1024
#define L2CAP_CONLESS_MTU   670

#define HCI_OPCODE(ogf,ocf)     ENDIAN_LE16(((ogf) << 10) | (ocf))

#define HCI_EVT_INQUIRY_COMPLETE            0x01
#define HCI_EVT_INQUIRY_RESULT              0x02
#define HCI_EVT_CONN_COMPLETE               0x03
#define HCI_EVT_CONN_REQUEST                0x04
#define HCI_EVT_DISCONN_COMPLETE            0x05
#define HCI_EVT_AUTH_COMPLETE               0x06
#define HCI_EVT_REMOTE_NAME_REQ_COMPLETE    0x07
#define HCI_EVT_ENC_CHANGE                  0x08
#define HCI_EVT_CHANGE_CONN_LK_COMPLETE     0x09
#define HCI_EVT_MASTER_LK_COMPLETE          0x0a
#define HCI_EVT_READ_REMOTE_FEAT_COMPLETE   0x0b
#define HCI_EVT_READ_REMOTE_VER_COMPLETE    0x0c
#define HCI_EVT_QOS_SETUP_COMPLETE          0x0d
#define HCI_EVT_COMMAND_COMPLETE            0x0e
#define HCI_EVT_COMMAND_STATUS              0x0f
#define HCI_EVT_HW_ERROR                    0x10
#define HCI_EVT_FLUSH_OCCURRED              0x11
#define HCI_EVT_ROLE_CHANGE                 0x12
#define HCI_EVT_NUM_COMP_PACKETS            0x13
#define HCI_EVT_MODE_CHANGE                 0x14
#define HCI_EVT_RETURN_LINK_KEYS            0x15
#define HCI_EVT_PIN_REQUEST                 0x16
#define HCI_EVT_LK_REQUEST                  0x17
#define HCI_EVT_LK_NOTIFICATION             0x18
#define HCI_EVT_LOOPBACK_COMMAND            0x19
#define HCI_EVT_BUFFER_OVERFLOW             0x1a
#define HCI_EVT_MAX_SLOTS_CHANGE            0x1b
#define HCI_EVT_READ_CLK_OFF_COMPLETE       0x1c
#define HCI_EVT_CONN_PACKET_TYPE_CHANGED    0x1d
#define HCI_EVT_QOS_VIOLATION               0x1e
#define HCI_EVT_PAGE_SCAN_MODE_CHANGE       0x1f
#define HCI_EVT_PAGE_SCAN_REP_MODE_CHANGE   0x20

typedef struct _evt_inquiry_result {
    unsigned char   num_responses;
    unsigned char   data[0];
} __attribute__ ((packed)) evt_inquiry_result;

typedef struct _evt_conn_complete {
    unsigned char   status;
    unsigned short  handle;
    bdaddr_t        bdaddr;
    unsigned char   link_type;
    unsigned char   encryption_mode;
} __attribute__ ((packed)) evt_conn_complete;

typedef struct _evt_conn_request {
    bdaddr_t        bdaddr;
    unsigned int    device_class:24;
    unsigned char   link_type;
} __attribute__ ((packed)) evt_conn_request;

typedef struct _evt_disconn_complete {
    unsigned char   status;
    unsigned short  handle;
    unsigned char   reason;
} __attribute__ ((packed)) evt_disconn_complete;

typedef struct _evt_command_complete {
    unsigned char   num_hci_cmd_packets;
    unsigned short  opcode;
    unsigned char   param[0];
} __attribute__ ((packed)) evt_command_complete;


typedef struct _evt_command_status {
    unsigned char   status;
    unsigned char   num_hci_cmd_packets;
    unsigned short  opcode;
} __attribute__ ((packed)) evt_command_status;

typedef struct _evt_page_scan_rep_mode_change {
    bdaddr_t        bdaddr;
    unsigned char   mode;
} __attribute__ ((packed)) evt_page_scan_rep_mode_change;


#define HCI_OGF_LINK_CONTROL    0x01

#define HCI_INQUIRY             0x01
typedef struct _cmd_inquiry {
    unsigned int    lap:24;
    unsigned char   length;
    unsigned char   num_responses;
} __attribute__ ((packed)) cmd_inquiry_t;

#define HCI_INQUIRY_CANCEL      0x02
#define HCI_PER_INQUIRY         0x03
typedef struct _cmd_per_inquiry {
    unsigned short  max_period;
    unsigned short  min_period;
    unsigned int    lap:24;
    unsigned char   length;
    unsigned char   num_responses;
} __attribute__ ((packed)) cmd_per_inquiry_t;

#define HCI_EXIT_PER_INQUIRY    0x04
#define HCI_CREATE_CONN         0x05
typedef struct _cmd_create_conn {
    bdaddr_t        bdaddr;
    unsigned short  pkt_type;
    unsigned char   pg_scan_rep_mode;
    unsigned char   pg_scan_mode;
    unsigned short  clk_offset;
    unsigned char   allow_role_switch;
} __attribute__ ((packed)) cmd_create_conn_t;

#define HCI_DISCONN             0x06
typedef struct _cmd_disconn {
    unsigned short  handle;
    unsigned char   reason;
} __attribute__ ((packed)) cmd_disconn_t;

#define HCI_ADD_SCO_CONN        0x07
#define HCI_ACCEPT_CONN_REQ     0x09
typedef struct _cmd_accept_conn_req {
    bdaddr_t        bdaddr;
    unsigned char   role;
} __attribute__ ((packed)) cmd_accept_conn_req_t;

#define HCI_REJECT_CONN_REQ     0x0a
typedef struct _cmd_reject_conn_req {
    bdaddr_t        bdaddr;
    unsigned char   reason;
} __attribute__ ((packed)) cmd_reject_conn_req_t;

#define HCI_REMOTE_NAME_REQ     0x19
typedef struct _cmd_remote_name_req {
    bdaddr_t        bdaddr;
    unsigned char   pg_scan_rep_mode;
    unsigned char   pg_scan_mode;
    unsigned short  clk_offset;
} __attribute__ ((packed)) cmd_remote_name_req_t;
typedef struct _cmdcmp_remote_name_req {
    unsigned char   status;
    bdaddr_t        bdaddr;
    unsigned char   remote_name[248];
} __attribute__ ((packed)) cmdcmp_remote_name_req_t;

#define HCI_OGF_HC_AND_BASEBAND 0x03

#define HCI_CHANGE_LOCAL_NAME   0x13
#define HCI_READ_LOCAL_NAME     0x14
typedef struct _cmdcmp_read_local_name {
    unsigned char   status;
    unsigned char   name[248];
} __attribute__ ((packed)) cmdcmp_read_local_name_t;

#define HCI_WRITE_SCAN_ENABLE   0x1a

#define SCAN_DISABLE                    0x00
#define SCAN_ENABLE_INQUIRY_ONLY        0x01
#define SCAN_ENABLE_PAGE_ONLY           0x02
#define SCAN_ENABLE_INQUIRY_AND_PAGE    0x03

#define HCI_HOST_BUFFER_SIZE    0x33

typedef struct _cmd_host_buffer_size {
    unsigned short  max_acl_len;
    unsigned char   max_sco_len;
    unsigned short  max_acl_packets;
    unsigned short  max_sco_packets;
} __attribute__ ((packed)) cmd_host_buffer_size_t;

#define HCI_OGF_INFO            0x04

#define HCI_READ_BUFFER_SIZE    0x05
typedef struct _cmdcmp_read_buffer_size {
    unsigned char   status;
    unsigned short  max_acl_len;
    unsigned char   max_sco_len;
    unsigned short  max_acl_packets;
    unsigned short  max_sco_packets;
} __attribute__ ((packed)) cmdcmp_read_buffer_size_t;

#define HCI_READ_BDADDR         0x09
typedef struct _cmdcmp_read_bdaddr {
    unsigned char   status;
    bdaddr_t        bdaddr;
} __attribute__ ((packed)) cmdcmp_read_bdaddr_t;

#define LAP_GENERAL_INQUIRY     0x9e8b33

#define PKT_TYPE_DM1            0x0008
#define PKT_TYPE_DH1            0x0010
#define PKT_TYPE_DM3            0x0400
#define PKT_TYPE_DH3            0x0800
#define PKT_TYPE_DM5            0x4000
#define PKT_TYPE_DH5            0x8000
#define PKT_TYPE_DEFAULT    (PKT_TYPE_DM1 | PKT_TYPE_DH1 | PKT_TYPE_DM3 | \
                             PKT_TYPE_DH3 | PKT_TYPE_DM5 | PKT_TYPE_DH5)

#define HCI_ERROR_LIMITED_RESOURCES     0x0d
#define HCI_ERROR_USER_ENDED            0x13

#define HCI_ACL_CONT            0x1
#define HCI_ACTIVE_BCAST        0x2
#define HCI_PNET_BCAST          0x4


#define ACL_CONT                0x1
#define ACL_FIRST               0x2

#define ACL_NO_BCAST            0x0
#define ACL_ACTIVE_BCAST        0x1
#define ACL_PNET_BCAST          0x2

#endif
